<?php

namespace Mbl\Analytics;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

add_action('admin_menu', 'Mbl\Analytics\admin_menu');
function admin_menu()
{
    $hook_suffix4 = add_submenu_page(
        'edit.php?post_type=wpm-page',
        __('Аналитика', 'mbl_admin'),
        __('Аналитика', 'mbl_admin'),
        'manage_options',
        'wpm-analytics',
        'Mbl\Analytics\admin_menu_page',
        6
    );
    add_action('load-' . $hook_suffix4, 'Mbl\Analytics\download_segment');
    add_action('load-' . $hook_suffix4, 'Mbl\Analytics\download_users');

    add_action('load-' . $hook_suffix4, function () {
        add_action('admin_enqueue_scripts', 'Mbl\Analytics\admin_assets', 100);
    });
}

function admin_menu_page()
{
    ?>
    <h1>Аналитика</h1>
    <div id="mban-root"></div>
    <?php
}

function admin_assets()
{
    wp_register_style(
        'mban-admin',
        MBAN_PLUGIN_URI . 'assets/analytics.css?v=' . time(),
        [],
        MBAN_VERSION
    );
    if (!empty($_GET['debugger'])) {
        wp_enqueue_style('mban-admin-common',
            MBAN_PLUGIN_URI . 'assets/admin.css',
            [],
            MBAN_VERSION
        );
    }
    wp_register_script(
        'mban-admin',
        MBAN_PLUGIN_URI . 'assets/analytics.js',
        [],
        MBAN_VERSION . ( getenv( 'MBL_DEV_MODE' ) == 1 ? time() : '')
    );
    wp_enqueue_style('mban-admin');
    wp_enqueue_script('mban-admin');
    $sources = mbl_access_level_sources();
    wp_localize_script('mban-admin', '__mban_ssr__', [
        'base_uri' => admin_url('edit.php?post_type=wpm-page&page=wpm-analytics'),
        'edit_uri' => admin_url('users.php?page=mbl_users_export'),
        'rest_route' => rest_url('mban/v1'),
        'nonce' => wp_create_nonce('wp_rest'),
        'levels' => array_values(
            get_terms([
                'taxonomy' => 'wpm-levels',
                'hide_empty' => false,
            ])
        ),
        'courses' => array_values(
            array_map(
                fn($term) => array_merge(
                    $term->to_array(),
                    ['levels' => array_map(fn($t) => $t->term_id, (new Course($term->term_id))->levels())]
                ),
                get_terms([
                    'taxonomy' => 'wpm-category',
                    'hide_empty' => false,
                ])
            )
        ),
        'segments' => (new Segments())->query(),
        'sources' => array_reduce(array_keys($sources), function ($carry, $key) use ($sources) {
            $carry[] = ['value' => $key, 'label' => $sources[$key]];
            return $carry;
        }, [])
    ]);
}

add_filter('script_loader_tag', '\Mbl\Analytics\defer_scripts', 10, 3);
function defer_scripts($tag, $handle, $src)
{
    if (preg_match('/^mban-.+$/', $handle) === 1) {
        return str_replace(' src', ' defer type="module" src', $tag);
    }

    return $tag;
}

function download_segment()
{
    if (empty($_GET['download'])) {
        return;
    }
    $id = (int)$_GET['download'];
    $segments = new Segments();
    $segmentData = $segments->byId($id);
    download_segment_users($segmentData);
}

function download_users()
{
    if (empty($_GET['downloadUsers'])) {
        return;
    }
    //viewUsers
    $segmentData = json_decode(base64_decode($_GET['downloadUsers']), true);
    download_segment_users($segmentData);
}

function download_segment_users($segmentData)
{
    global $wpdb;
    $segment = new Segment($segmentData['type'], $segmentData['params']);
    try {
        $fields = "u.ID, u.user_login AS userName, u.user_email AS email";
        $users = $wpdb->get_results(
            "SELECT $fields FROM $wpdb->users AS u WHERE u.ID IN ({$segment->selection()->idsRequest()})",
            ARRAY_A
        );
        $metaKeys = ['first_name', 'last_name', 'surname', 'phone'];
        $metaKeysIn = implode(',', array_map(fn($key) => "'$key'", $metaKeys));
        $usersMetaFields = $wpdb->get_results(
            "SELECT user_id, meta_key, meta_value FROM $wpdb->usermeta AS um
                WHERE um.user_id IN ({$segment->selection()->idsRequest()})
                  AND um.meta_key IN($metaKeysIn) AND meta_value != '' AND meta_value IS NOT NULL",
            ARRAY_A
        );
        $usersMetaFieldsByUserId = [];
        foreach ($usersMetaFields as $meta) {
            $usersMetaFieldsByUserId[$meta['user_id']][$meta['meta_key']] = $meta['meta_value'];
        }
        $data = [['ID', 'Логин', 'Фамилия', 'Имя', 'Отчество', 'Телефон', 'Email']];
        foreach ($users as $user) {
            $data[] = [
                $user['ID'],
                $user['userName'],
                $usersMetaFieldsByUserId[$user['ID']]['last_name'] ?? '',
                $usersMetaFieldsByUserId[$user['ID']]['first_name'] ?? '',
                $usersMetaFieldsByUserId[$user['ID']]['surname'] ?? '',
                $usersMetaFieldsByUserId[$user['ID']]['phone'] ?? '',
                $user['email']
            ];
        }
        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();;
        foreach (['A', 'B', 'C', 'D', 'E', 'F', 'G'] as $heading) {
            $sheet->getColumnDimension($heading)->setAutoSize(true);
        }
        $sheet->fromArray($data);
        $writer = new Xlsx($spreadsheet);
        header("Expires: Mon, 1 Apr 1974 05:00:00 GMT");
        header("Last-Modified: " . gmdate("D,d M YH:i:s") . " GMT");
        header("Cache-Control: no-cache, must-revalidate");
        header("Pragma: no-cache");
        header("Content-type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        header("Content-Disposition: attachment; filename=" . "users.xlsx");
        $writer->save('php://output');
    } catch (\Exception $e) {

    }
    die;
}
