<?php

namespace Mbl\Analytics;

use function register_rest_route;

add_action('rest_api_init', 'Mbl\Analytics\rest_api');

function rest_api()
{
    if (getenv( 'MBL_DEV_MODE' ) == 1) {
        error_reporting(E_ALL);
        ini_set('display_errors', 1);
        header("Access-Control-Allow-Origin: *");
    }
    $permission_cb = fn() => getenv( 'MBL_DEV_MODE' ) == 1 || current_user_can( 'manage_options' );

    register_rest_route('mban/v1', '/sources/', [
        'methods' => 'POST',
        'callback' => [new SourcesRoute(), 'response'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/courses/', [
        'methods' => 'POST',
        'callback' => [new CourseStatRoute(), 'response'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/expiring/', [
        'methods' => 'POST',
        'callback' => [new ExpiringRoute(), 'response'],
        'permission_callback' => $permission_cb
    ]);
    $salesRoute = new SalesRoute();
    register_rest_route('mban/v1', '/sales/', [
        'methods' => 'POST',
        'callback' => [$salesRoute, 'response'],
        'permission_callback' => $permission_cb
    ]);
    $segmentRoute = new SegmentsRoute();
    register_rest_route('mban/v1', '/segments/create', [
        'methods' => 'POST',
        'callback' => [$segmentRoute, 'create'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/segments/edit', [
        'methods' => 'POST',
        'callback' => [$segmentRoute, 'edit'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/segments/delete', [
        'methods' => 'POST',
        'callback' => [$segmentRoute, 'delete'],
        'permission_callback' => $permission_cb
    ]);
    $PreSegmentsRoute = new PreSegmentsRoute();
    register_rest_route('mban/v1', '/segments/view', [
        'methods' => 'GET',
        'callback' => [$PreSegmentsRoute, 'view'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/segments/view/(?P<id>\d+)', [
        'methods' => 'GET',
        'callback' => [$PreSegmentsRoute, 'segment'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/segments/edit-ui/(?P<id>\d+)', [
        'methods' => 'POST',
        'callback' => [$PreSegmentsRoute, 'edit'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/segments/create-ui', [
        'methods' => 'POST',
        'callback' => [$PreSegmentsRoute, 'create'],
        'permission_callback' => $permission_cb
    ]);
    register_rest_route('mban/v1', '/utm', [
        'methods' => 'GET',
        'callback' => [new RtUtm(), 'response'],
        'permission_callback' => $permission_cb
    ]);
}
