import * as style from './App.module.css';
import React from "react";
import Tabs from "./components/Tabs";
import { StateContent } from "./state/models/StateContent";
import { Context } from "./state/state";
import { useReducer } from "react";
import TabContent from "./components/TabContent";
import Sales from "./components/Sales";
import SourcesTab from "./components/SourcesTab";
import Sources from "./src/Sources";
import { initialState } from "./state/initialState";
import reducer from "./state/reducer";
import Expiring from "./components/Expiring";
import SegmentEvents from "./src/SegmentEvents";
import ExpiringEvents from "./src/ExpiringEvents";
import SalesEvents from "./src/SalesEvents";
import CoursesTab from "./components/CoursesTab";
import CoursesEvents from "./src/CoursesEvents";
import Segments from "./components/Segments";
import Popup from "./components/Popup";
import Success from "./components/Success";
import { AppState, StateWithoutSsr } from "./state/models/AppState";
import SegmentTableEvents from "./src/SegmentTableEvents";

export function App(props) {
    const [state, dispatch] = useReducer(reducer, initialState, (initial: StateWithoutSsr): AppState => {
        return { ...initial, ssr: props }
    });
    const value: StateContent = {
        state: state,
        actions: {
            toggleTab: (name) => {
                dispatch({ type: 'changeTab', payload: name });
            },
            segment: new SegmentEvents(dispatch, props),
            segmentTable: new SegmentTableEvents(dispatch, props),
            courses: new CoursesEvents(dispatch),
            sources: new Sources(dispatch),
            expiring: new ExpiringEvents(dispatch),
            sales: new SalesEvents(dispatch, props),
            fetch: async ({ route, method = 'GET', body, headers = {} }) => {
                const response = await fetch(`${props.rest_route}/${route}`, {
                    method,
                    headers: {
                        "Content-Type": "application/json",
                        'X-WP-Nonce': props.nonce,
                        ...headers,
                    },
                    body: JSON.stringify(body)
                })
                return await response.json();
            }
        }
    };

    return <Context.Provider value={value}>
        {state.segmentSuccess &&
            <Success>Сохранено!</Success>
        }
        <Tabs/>
        <TabContent name="sources" component={SourcesTab}/>
        <TabContent name="courses" component={CoursesTab}/>
        <TabContent name="sales" component={Sales}/>
        <TabContent name="expiring" component={Expiring}/>
        <TabContent name="segments" component={Segments}/>
        {state.segmentPopup &&
            <Popup/>
        }
    </Context.Provider>
}
