import React, { useContext } from "react";
import * as style from './Expiring.module.css';
import DatePicker from "react-datepicker";
import ru from 'date-fns/locale/ru';
import { forwardRef } from "react";
import { Context } from "../../state/state";
import { Unit } from "../../state/models/Unit";
import Spinner from "../Spinner"
import SegmentBtn from "../SegmentBtn";
import SegmentSwitch from "../SegmentSwitch";
import SelectSegment from "../SelectSegment/SelectSegment";
import UserLink from "../UserLink";

export default function Expiring() {
    const { state, actions } = useContext(Context)
    const { ssr } = state
    const {
        warning,
        disabled,
        dateTo,
        dateFrom,
        level,
        items,
        segment,
        total_segment,
        is_segment,
        segment_tab
    } = state.expiring
    const { warningChanged, itemsFetching, itemsFetched, dateToChanged, dateFromChanged } = actions.expiring
    const { levelChanged, switchedSegmentTab, segmentChange } = actions.expiring

    async function submit() {
        warningChanged('')
        let body;
        let is_segment = segment_tab === '2'
        if (segment_tab === '2' && segment === '') {
            warningChanged('Укажите сегмент')
            return;
        }
        if (level === "") {
            warningChanged('Укажите уровень доступа')
            return;
        } else if (dateFrom === null || dateTo === null) {
            warningChanged('Укажите дату')
            return;
        }
        if (segment_tab === '1') {
            body = { dateTo, dateFrom, level }
        } else {
            body = { dateTo, dateFrom, level, segment }
        }
        try {
            itemsFetching()
            let response = await actions.fetch({
                route: 'expiring',
                method: 'POST',
                body
            })
            if (response.ok) {
                itemsFetched({ items: response.items, total_segment: response.segment, is_segment })
            } else {

            }

        } catch (e) {
            console.log(e)
        }
    }

    function renderUnit(unit: Unit) {
        return { months: 'мес.', days: 'дн.' }[unit]
    }

    const total = items.reduce((acc, item) => acc + +item.count, 0)

    function downloadUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.base_uri + '&downloadUsers=' + encodeURIComponent(params))
    }

    function viewUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.edit_uri + '&viewUsers=' + encodeURIComponent(params))
    }

    function showSegment(value) {
        return !is_segment && value !== 0 && value.toString().trim() !== '-'
    }

    return <div>
        {!!warning &&
            <div className={`notice notice-warning ${style.error}`}>
                <p>{warning}</p>
            </div>
        }
        <div className={style.wrap}>
            <div className={style.head}>
                <SegmentSwitch
                    label_first="Все студенты"
                    active={segment_tab}
                    onChange={switchedSegmentTab}
                />
                {segment_tab === '2' && <SelectSegment value={segment} onChange={segmentChange}/>}
                <div className={style.dates}>
                    <span className="dashicons dashicons-calendar-alt"></span>&nbsp;Доступ заканчивается в
                    период
                    c&nbsp;<DatePicker
                    placeholderText="__ /__ /____"
                    locale={ru}
                    dateFormat="dd.MM.yyyy"
                    customInput={<DateInput/>}
                    selected={dateFrom}
                    onChange={dateFromChanged}
                />&nbsp;по&nbsp;
                    <DatePicker
                        placeholderText="__ /__ /____"
                        locale={ru}
                        dateFormat="dd.MM.yyyy"
                        customInput={<DateInput/>}
                        selected={dateTo}
                        onChange={dateToChanged}
                    />
                </div>
                <div className={style.level}>
                    <label htmlFor="level"><span className="dashicons dashicons-lock"></span>&nbsp;Уровень
                        доступа</label>
                    &nbsp;<select className={style.levelSelect} name="level" id="level"
                                  value={level.term_id}
                                  onChange={levelChanged}
                >
                    <option value="">Выбрать</option>
                    {ssr.levels.map(level =>
                        <option key={level.slug} value={level.term_id}>{level.name}</option>
                    )}
                </select>
                </div>
                <div>
                    <button disabled={disabled}
                            onClick={submit}
                            className="button button-primary">Применить
                    </button>
                </div>

            </div>
            {disabled
                ? <div className={style.spinnerWrap}><Spinner/></div>
                : <table className={style.table}>
                    <thead>
                    <tr>
                        <td>Название уровня доступа</td>
                        <td>Время доступа</td>
                        <td>Пользователи у которых закончиться доступ</td>
                    </tr>
                    </thead>
                    <tbody>
                    {items.map((item, index) =>
                        <tr key={index}>
                            <td>{ssr.levels.find(item => +item.term_id === +level).name}</td>
                            <td>{item.duration} {renderUnit(item.units)}</td>
                            <td>
                                {showSegment(item.count) ? <>
                                    <UserLink onClick={() => viewUsers(item.segment)}>{item.count}</UserLink>&nbsp;
                                    <SegmentBtn data={item.segment} onClick={downloadUsers}/>
                                </> : item.count}
                            </td>
                        </tr>
                    )}
                    </tbody>
                    {total_segment &&
                        <tfoot>
                        <tr>
                            <td colSpan={2}>Всего уникальных пользователей</td>
                            <td>
                                {showSegment(total) ? <>
                                    <UserLink onClick={() => viewUsers(total_segment)}>{total}</UserLink>&nbsp;
                                    <SegmentBtn data={total_segment} onClick={downloadUsers}/>
                                </> : total}
                            </td>
                        </tr>
                        </tfoot>
                    }
                </table>
            }
        </div>
    </div>
}

const DateInput = forwardRef((props, ref) => {
    return (
        <input type="text" style={{ width: '100px' }} {...props} ref={ref}/>
    )
});
