import * as style from './MultiSelect.module.css';
import React, { useEffect, useState, useRef } from "react";

export default function MultiSelect({ name, options, value, onChange, allAllowed = true, disabled = false }) {
    const [state, setState] = useState(false)
    const wrapperRef = useRef(null);
    let allChecked = value.indexOf('all') === 0;

    function onChangeCheck(event) {
        let result = [];
        if (event.target.checked) {
            if (value.indexOf(event.target.value) === -1) {
                result = [...value, event.target.value]
            }
        } else {
            if (value.indexOf('all') === 0) {
                result = options.map(item => item.val).filter(val => val !== event.target.value)
            } else {
                result = value.filter(val => val !== event.target.value)
            }
        }
        if (result.length === options.length) {
            if (allAllowed) {
                result = ['all']
            } else {
                result = [...value]
            }
        }

        onChange(result)
    }

    function onSelectAll(event) {
        if (event.target.checked) {
            onChange(['all'])
        } else {
            onChange([])
        }
    }

    useEffect(() => {
        function handleClickOutside(event) {
            if (wrapperRef.current && !wrapperRef.current.contains(event.target)) {
                setState(false);
            }
        }

        document.addEventListener("mousedown", handleClickOutside);
        return () => {
            document.removeEventListener("mousedown", handleClickOutside);
        };
    }, [wrapperRef]);

    function onClick() {
        if(disabled) {
            return false;
        }
        setState(state => !state)
    }
    return <div className={style.root} ref={wrapperRef}>
        <div className={`${style.select}${disabled ? ` ${style.selectDisabled}` : ''}`} onClick={onClick}>
            {allChecked
                ? 'Все'
                : <span>{value.length > 0 ? `Выбрано ${value.length}` : 'Выбрать'}</span>
            }
            <span className={`dashicons dashicons-arrow-down-alt2 ${style.icon}`}></span>
        </div>
        {state &&
            <div className={style.dropdown}>
                {allAllowed &&
                    <label>
                        <input onChange={onSelectAll}
                               name={name}
                               value="all"
                               type="checkbox"
                               checked={allChecked}
                        /> Все
                    </label>
                }
                {options.map(({ val, label }, key) =>
                    <label key={key}>
                        <input onChange={onChangeCheck}
                               name={name}
                               value={val}
                               type="checkbox"
                               checked={value.indexOf(val) !== -1 || allChecked}
                        /> {label}
                    </label>
                )}
            </div>
        }
    </div>
}
