import React, { useContext } from "react";
import * as style from "././Popup.module.css";
import Spinner from "../Spinner";
import { Context } from "../../state/state";

export default function Popup() {
    const { actions, state } = useContext(Context)
    const { segment, segmentSaving, segmentError, nameInvalid, descriptionInvalid } = state
    const { closeSegmentPopup, saving, saved, savingComplete, onError, onValidation } = actions.segment
    const { changeName, changeDescription } = actions.segment

    async function onSubmit() {
        onValidation()
        if (segment.name.trim().length === 0 || segment.description.trim().length === 0) return;
        saving()
        try {
            let response = await actions.fetch({
                route: 'segments/create',
                method: 'POST',
                body: segment
            });
            if (!response.ok) {
                throw new Error()
            }
            saved(response)
        } catch (error) {
            onError('Произошла ошибка. Повторите позже')
        } finally {
            savingComplete()
        }
    }

    return <div className={style.root}>
        <div className={`${style.body} ${segmentSaving ? style.blocking : ''}`}>
            {segmentSaving &&
                <div className={style.spinner}>
                    <Spinner/>
                </div>
            }
            <button type={`button`} onClick={closeSegmentPopup} className={style.close}>
                <span className="dashicons dashicons-no-alt"></span></button>
            {segmentError.length !== 0 &&
                <div style={{ marginLeft: 0, marginRight: 0 }} className={`notice notice-error`}>
                    <p>{segmentError}</p>
                </div>
            }
            <div className={style.name}>
                <label htmlFor="segmentName" style={{ display: 'none' }}>Название сегмента</label>
                <div>
                    <input className={style.inp}
                           value={segment.name}
                           onChange={changeName}
                           type="text" name="name" id="segmentName"
                           placeholder={`Название сегмента`}/>
                    {nameInvalid &&
                        <p className={style.error}>Заполните поле</p>
                    }
                </div>
            </div>
            <div>
                <label htmlFor="segmentdescription" style={{ display: 'none' }}>Описание</label>
                <div>
                    <textarea className={style.textarea} name="description" id="segmentdescription"
                              value={segment.description}
                              onChange={changeDescription}
                              placeholder={`Описание`}
                              cols="30" rows="8"></textarea>
                    {descriptionInvalid &&
                        <p className={style.error}>Заполните поле</p>
                    }
                </div>
            </div>
            <div className={style.btnWrap}>
                <button type={`button`}
                        onClick={onSubmit}
                        className={`button button-primary ${style.btn}`}>Сохранить
                </button>
            </div>
        </div>
        <div className={style.overlay}></div>
    </div>
}