import React, { forwardRef, useContext } from "react";
import * as style from "./Sales.module.css"
import { Context } from "../../state/state";
import MultiSelect from "../MultiSelect"
import DatePicker from "react-datepicker";
import ru from "date-fns/locale/ru";
import Spinner from "../Spinner"
import SegmentBtn from "../SegmentBtn";
import SegmentSwitch from "../SegmentSwitch";
import SelectSegment from "../SelectSegment/SelectSegment";
import UserLink from "../UserLink";

export default function Sales() {
    const { actions, state } = useContext(Context)
    const { ssr } = state
    const { dateTo, dateFrom, level, levels, warning, isFetching } = state.sales
    const { sale, no_sale, sale_segment, no_sale_segment, segment_tab, segment, is_segment } = state.sales
    const { levelsChanged, dateFromChanged, dateToChanged, levelChanged } = actions.sales
    const { onFetching, onFetched, warningChanged, switchedSegmentTab, segmentChange } = actions.sales


    async function submit() {
        let body;
        let is_segment = segment_tab === '2'
        if (level.length === 0) {
            warningChanged('Заполните "Уровень доступа", который купили/не купили')
            return;
        }
        if (segment_tab === '1') {
            if (levels.length === 0) {
                warningChanged('Выберите хотябы один "Уровень доступа"')
                return;
            }
            body = { dateTo, dateFrom, levels, level }
        } else {
            body = { segment, level }
        }
        onFetching()
        try {
            let response = await actions.fetch({
                route: 'sales',
                method: 'POST',
                body
            })
            if (response.ok) {
                onFetched({ ...response, is_segment })
            } else {

            }
        } catch (e) {
            console.log(e)
        }
    }

    function downloadUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.base_uri + '&downloadUsers=' + encodeURIComponent(params))
    }

    function viewUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.edit_uri + '&viewUsers=' + encodeURIComponent(params))
    }

    return <div>
        {!!warning &&
            <div className={`notice notice-warning ${style.error}`}>
                <p>{warning}</p>
            </div>
        }
        <SegmentSwitch active={segment_tab} onChange={switchedSegmentTab}/>
        {segment_tab === '1'
            ? <div className={style.row}>
                <div className={style.left}>
                    <div className={style.level}>
                        <div>
                            <span className="dashicons dashicons-lock"></span> Уровень доступа
                        </div>
                        <MultiSelect
                            allAllowed={false}
                            value={levels}
                            onChange={levelsChanged}
                            name="level"
                            options={ssr.levels.map(level => ({ val: level.term_id.toString(), label: level.name }))}
                        />
                    </div>
                    <div className={style.dates}>
                   <span>
                        <span className="dashicons dashicons-calendar-alt"></span> Доступ получен в период c&nbsp;
                   </span>
                        <DatePicker
                            placeholderText="__ /__ /____"
                            locale={ru}
                            dateFormat="dd.MM.yyyy"
                            customInput={<DateInput/>}
                            selected={dateFrom}
                            onChange={dateFromChanged}
                        />
                        &nbsp;по&nbsp;
                        <DatePicker
                            placeholderText="__ /__ /____"
                            locale={ru}
                            dateFormat="dd.MM.yyyy"
                            customInput={<DateInput/>}
                            selected={dateTo}
                            onChange={dateToChanged}
                        />
                    </div>
                </div>
            </div>
            : <SelectSegment value={segment} onChange={segmentChange}/>
        }
        <div className={style.brnWrap}>
            <button type="button" className="button button-primary" onClick={submit}>Применить</button>
        </div>
        <div className={style.table}>
            <div className={style.tableTop}>
                <label htmlFor="level" className={style.levelLabel}>Уровень доступа</label>
                <select name="level" id="level" value={level} onChange={levelChanged}>
                    <option value="">Выбрать</option>
                    {ssr.levels.filter(item => !levels.includes(item.term_id.toString())).map(level =>
                        <option key={level.slug} value={level.term_id}>{level.name}</option>
                    )}
                </select>
            </div>
            {isFetching
                ? <div style={{ textAlign: 'center', padding: '15px' }}><Spinner/></div>
                : <>
                    <div className={style.col}>
                        <div>Купили (получили доступ)</div>
                        <div>Не купили (не получили доступ)</div>
                    </div>
                    <div className={style.col}>
                        <div>
                            {!!sale && !is_segment ?
                                <>
                                    <UserLink onClick={() => viewUsers(sale_segment)}>{sale}</UserLink>
                                    &nbsp;
                                    <SegmentBtn data={sale_segment} onClick={downloadUsers}/>
                                </>
                                : sale
                            }
                        </div>
                        <div>
                            {!!no_sale && !is_segment ?
                                <>
                                    <UserLink onClick={() => viewUsers(no_sale_segment)}>{no_sale}</UserLink>
                                    &nbsp;
                                    <SegmentBtn data={no_sale_segment} onClick={downloadUsers}/>
                                </>
                                : no_sale
                            }
                        </div>
                    </div>
                </>}
        </div>
    </div>
}

const DateInput = forwardRef((props, ref) => {
    return (
        <input type="text" style={{ width: '100px' }} {...props} ref={ref}/>
    )
});