import React, { useContext } from "react";
import * as style from './Segments.module.css'
import { Context } from "../../state/state";
import SegmentRemoveBtn from "../SegmentRemoveBtn";
import Spinner from "../Spinner";
import ErrorNotice from "../ErrorNotice";
import SuccessNotice from "../SuccessNotice";

export default function Segments() {
    const { state, actions } = useContext(Context)
    const { ssr: { base_uri, segments, edit_uri } } = state
    const { blocking, completed, error } = state.segmentTable

    async function onTrashClick(id: number) {
        actions.segmentTable.blocked(true)
        try {
            let body = { id }
            let response = await actions.fetch({
                route: 'segments/delete',
                method: 'POST',
                body
            })

            if (response.ok) {
                actions.segmentTable.onCompleted()
                actions.segmentTable.onRemoveSegment(id)
            } else {
                actions.segmentTable.onFailed(response.message)
            }
        } catch (e) {
            actions.segmentTable.onFailed('Произошла ошибка. Попробуйте позже')
        } finally {
            actions.segmentTable.blocked(false)
        }
    }

    function onCloseSuccess(event) {
        actions.segmentTable.onCloseSuccessNotice()
    }

    var editSegment = function (id) {
        return edit_uri + '&viewSegment=' + id
    }

    return <div className={style.root}>
        {error && <ErrorNotice message="Произошла ошибка"/>}
        {completed && (<div className={style.success}>
            <SuccessNotice onClose={onCloseSuccess}>Успешно удален</SuccessNotice>
        </div>)}
        {blocking && (<div className={style.overlay}><Spinner/></div>)}
        <table border={1} className={style.table}>
            <thead>
            <tr>
                <th>Название сегмента</th>
                <th>Описание</th>
                <th></th>
                <th></th>
                <th></th>
                {/*<th></th>*/}
            </tr>
            </thead>
            <tbody>
            {segments.map(segment =>
                <tr key={segment.id}>
                    <td>{segment.name}</td>
                    <td>{segment.description}</td>
                    <td>
                        <a className={style.btn} target="_blank" rel="nofollow"
                           href={`${base_uri}&download=${segment.id}`}>
                            <span className="dashicons dashicons-download"></span>
                        </a>
                    </td>
                    {/*<td>*/}
                    {/*    <button className={style.btn} type={`button`} onClick={editSegment}>*/}
                    {/*        <span className="dashicons dashicons-edit"></span>*/}
                    {/*    </button>*/}
                    {/*</td>*/}
                    <td>
                        <a className={style.btn} target="_blank" rel="nofollow" href={editSegment(segment.id)}>
                            <span className="dashicons dashicons-edit"></span>
                        </a>
                    </td>
                    <td>
                        <SegmentRemoveBtn segment={segment} onClick={onTrashClick} className={style.btn}/>
                    </td>
                </tr>
            )}
            </tbody>
        </table>
    </div>
}