import React, { useCallback, useEffect, useState } from "react";
import * as style from './Select.module.css'
import { useRef } from "react/index";
import Spinner from '../Spinner'

export default function Select({ id, value, onChange, onGet }) {
    var ref = useRef(null)
    var inpRef = useRef(null)
    var [show, setShow] = useState(false);
    var [loading, setLoading] = useState(false);
    var [inputValue, setInputValue] = useState('');
    var [results, setResults] = useState([]);

    var toggle = () => {
        setShow(val => !val)
    }

    var handleChange = (event) => {
        setInputValue(event.target.value);
    };

    var handleSelect = (selectedValue) => {
        setInputValue('');
        onChange(selectedValue);
        setShow(false);
    };

    var handleGet = useCallback(
        debounce((query) => {
            setLoading(true)
            setResults([])
            onGet(id, query).then(data => {
                setResults(data)
                setLoading(false)
            });

        }, 500),
        [onGet, setResults]
    );

    var onClear = () => {
        onChange('')
    };

    useEffect(() => {
        handleGet(inputValue);
    }, [inputValue]);

    useEffect(() => {
        var clickOutside = (event) => {
            if (ref.current && !ref.current.contains(event.target)) {
                setShow(false)
            }
        }

        document.addEventListener('click', clickOutside)

        return () => document.removeEventListener('click', clickOutside)
    }, [ref])

    useEffect(() => {
        if (!show || !inpRef.current) {
            return
        }

        inpRef.current.focus()
    }, [show])

    return <div ref={ref} className={style.container}>
        <div onClick={toggle}>
            <div className={style.selection}>
                <div className={style.value}>{value}</div>
                <div role="presentation" className={style.arrowWrap}>
                    <span className={style.arrow}></span>
                </div>
            </div>
        </div>
        {show && (
            <div className={style.dropdown}>
                <input ref={inpRef}
                       className={style.search}
                       id={id}
                       value={inputValue}
                       onChange={handleChange}
                       type="search"
                       spellCheck="false" role="searchbox" autoCorrect="off" autoComplete="off"/>
                {!!value.length && (
                    <button onClick={onClear} className={style.clear} type="button">очистить</button>
                )}
                {loading && (
                    <div className={style.spinner}>
                        <Spinner />
                    </div>
                )}
                <ul className={style.results}>
                    {results.map((result) => (
                        <li key={result.id} className={style.option}
                            onClick={() => handleSelect(result.label)}>{result.label}</li>
                    ))}
                </ul>
            </div>
        )}
    </div>
}

function debounce(func, ms) {
    var timeout;
    return function (...args: any[]) {
        clearTimeout(timeout);
        timeout = setTimeout(function () {
            func.apply(this, args)
        }, ms);
    };
}