import React from "react";
import * as style from "./Sources.module.css";
import { Context } from "../../state/state";
import { forwardRef, useContext } from "react";
import DatePicker from "react-datepicker";
import "react-datepicker/dist/react-datepicker.css";
import ru from 'date-fns/locale/ru';
import MultiSelect from "../MultiSelect";
import Spinner from "../Spinner";
import SegmentBtn from "../SegmentBtn";
import UserLink from "../UserLink";
import Select from "../Select";

export default function Sources() {
    const { state: { ssr, sources }, actions } = useContext(Context)
    const { start, end, source, terms, duration, warning, isFetching, items, total_segment, total } = sources
    const { startChanged, endChanged, sourceChanged, termsChanged, durationChanged } = actions.sources
    const { warningChanged, onFetching, onFetched } = actions.sources
    const { utm_source, utm_medium, utm_campaign, utm_id, utm_term, utm_content } = sources
    const { utmSourceChanged, utmMediumChanged, utmCampaignChanged, utmIdChanged, utmTermChanged } = actions.sources
    const { utmContentChanged } = actions.sources

    function downloadUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.base_uri + '&downloadUsers=' + encodeURIComponent(params))
    }

    function viewUsers(value) {
        var params = btoa(JSON.stringify(value))
        window.open(ssr.edit_uri + '&viewUsers=' + encodeURIComponent(params))
    }

    async function submit() {
        warningChanged('')
        if (source.length === 0) {
            warningChanged('Укажите пособы получения')
            return;
        }
        if (terms.length === 0) {
            warningChanged('Укажите уровни доступа')
            return;
        }
        try {
            onFetching()
            let data = await actions.fetch({
                route: 'sources',
                method: 'POST',
                body: {
                    start, end, source, terms, duration,
                    utm_source, utm_medium, utm_campaign, utm_id, utm_term, utm_content
                },
            })
            if (data.ok) {
                onFetched(data.data)
            }
        } catch (e) {
            console.log(e)
        }
    }

    var fetchUtm = async (utm_name, query) => {
        if (!query.trim().length) {
            return [];
        }

        var response = await actions.fetch({
            route: `utm?utm_name=${encodeURIComponent(utm_name)}&search=${encodeURIComponent(query)}`
        })

        return response.items.map(i => ({ id: i, label: i }));
    }

    return <div>
        {!!warning &&
            <div className={`notice notice-warning ${style.error}`}>
                <p>{warning}</p>
            </div>
        }
        <div className={style.dates}>
            <div>
                <span className={style.lbl}><span
                    className={`dashicons dashicons-calendar-alt ${style.dateIcon}`}></span><span>&nbsp;Период с&nbsp;</span></span><DatePicker
                selected={start}
                onChange={startChanged}
                locale={ru}
                dateFormat="dd.MM.yyyy"
                customInput={<DateInput/>}
            />
            </div>
            <div>
                &nbsp;по&nbsp;<DatePicker
                selected={end}
                onChange={endChanged}
                locale={ru}
                dateFormat="dd.MM.yyyy"
                customInput={<DateInput/>}
            />
            </div>
        </div>
        <div className={style.btnWrap}>
            <button onClick={submit} disabled={isFetching} className="button button-primary"
                    type="button">Применить
            </button>
        </div>
        {isFetching && <Spinner/>}
        <table className={style.table}>
            <thead>
            <tr>
                <th>
                    <label htmlFor="">Способы получения доступа</label>
                    <MultiSelect
                        value={source}
                        onChange={sourceChanged}
                        options={ssr.sources.map(({ value, label }) => ({ val: value, label }))}
                        name="source"
                    />
                </th>
                <th>
                    <label htmlFor="">Уровни доступа</label>
                    <MultiSelect
                        value={terms}
                        onChange={termsChanged}
                        options={ssr.levels.map(({ term_id, name }) => ({ val: term_id.toString(), label: name }))}
                        name="terms"
                    />
                </th>
                <th>
                    <label htmlFor="duration">Время доступа</label>
                    <div>
                        <select name="duration" id="duration" value={duration} onChange={durationChanged}>
                            <option value="all">Все</option>
                            <option value="limited">Ограничено</option>
                            <option value="unlimited">Не ограничено</option>
                        </select>
                    </div>
                </th>
                <th>
                    <label htmlFor="utm_source">utm_source</label>
                    <Select id="utm_source" onGet={fetchUtm} value={utm_source} onChange={utmSourceChanged}/>
                </th>
                <th>
                    <label htmlFor="utm_medium">utm_medium</label>
                    <Select id="utm_medium" onGet={fetchUtm} value={utm_medium} onChange={utmMediumChanged}/>
                </th>
                <th>
                    <label htmlFor="utm_campaign">utm_campaign</label>
                    <Select id="utm_campaign" onGet={fetchUtm} value={utm_campaign} onChange={utmCampaignChanged}/>
                </th>
                <th>
                    <label htmlFor="utm_id">utm_id</label>
                    <Select id="utm_id" onGet={fetchUtm} value={utm_id} onChange={utmIdChanged}/>
                </th>
                <th>
                    <label htmlFor="utm_term">utm_term</label>
                    <Select id="utm_term" onGet={fetchUtm} value={utm_term} onChange={utmTermChanged}/>
                </th>
                <th>
                    <label htmlFor="utm_content">utm_content</label>
                    <Select id="utm_content" onGet={fetchUtm} value={utm_content} onChange={utmContentChanged}/>
                </th>
                <th>
                    Пользователи
                    <div style={{ height: '30px' }}></div>
                </th>
            </tr>
            </thead>
            <tbody>
            {items.map((item, index) =>
                <React.Fragment key={index}>
                    {item['items'].map((el, ind) =>
                        <tr key={ind} className={style.tr}>
                            <td>{el.source.name}</td>
                            <td>{item.name}</td>
                            <td>{el.duration_d}</td>
                            <td>{el.utm_source}</td>
                            <td>{el.utm_medium}</td>
                            <td>{el.utm_campaign}</td>
                            <td>{el.utm_id}</td>
                            <td>{el.utm_term}</td>
                            <td>{el.utm_content}</td>
                            <td>
                                <UserLink onClick={() => viewUsers(el.segment)}>{el.count}</UserLink>&nbsp;
                                {!!el.count &&
                                    <SegmentBtn data={el.segment}
                                                onClick={downloadUsers}
                                    />
                                }
                            </td>
                        </tr>
                    )}
                </React.Fragment>
            )}
            {items.length === 0
                ? <tr>
                    <td style={{ textAlign: 'center' }} colSpan={10}>По данному запросу не найдено результатов</td>
                </tr>
                : <tr>
                    <td colSpan={9}>Всего уникальных пользователей</td>
                    <td>
                        <UserLink onClick={() => viewUsers(total_segment)}>{total}</UserLink>&nbsp;
                        {!!total &&
                            <SegmentBtn data={total_segment}
                                        onClick={downloadUsers}
                            />
                        }
                    </td>
                </tr>
            }
            </tbody>
        </table>
    </div>
}

const DateInput = forwardRef((props, ref) => {
    return (
        <input type="text" style={{ width: '100px' }} {...props} ref={ref}/>
    )
});
