import { AppState } from "./models/AppState";

export default function reducer(state: AppState, action) {
    let newState: AppState;
    switch (action.type) {
        case 'changeTab':
            newState = { ...state, activeTab: action.payload }
            break;
        case 'startChanged':
            if (action.payload > state.sources.start) {
                newState = {
                    ...state,
                    sources: {
                        ...state.sources,
                        start: action.payload,
                        end: action.payload,
                        warning: ''
                    }
                }
            } else {
                newState = { ...state, sources: { ...state.sources, start: action.payload, warning: '' } }
            }
            break;
        case 'endChanged':
            if (action.payload < state.sources.start) {
                newState = {
                    ...state,
                    sources: {
                        ...state.sources,
                        start: action.payload,
                        end: action.payload,
                        warning: ''
                    }
                }
            } else {
                newState = { ...state, sources: { ...state.sources, end: action.payload, warning: '' } }
            }
            break;
        case 'sourceChanged':
            newState = { ...state, sources: { ...state.sources, source: action.payload } }
            break;
        case 'termsChanged':
            newState = { ...state, sources: { ...state.sources, terms: action.payload } }
            break;
        case 'durationChanged':
            newState = { ...state, sources: { ...state.sources, duration: action.payload } }
            break;
        case 'utmSourceChanged':
            newState = { ...state, sources: { ...state.sources, utm_source: action.payload } }
            break;
        case 'utmMediumChanged':
            newState = { ...state, sources: { ...state.sources, utm_medium: action.payload } }
            break;
        case 'utmCampaignChanged':
            newState = { ...state, sources: { ...state.sources, utm_campaign: action.payload } }
            break;
        case 'utmIdChanged':
            newState = { ...state, sources: { ...state.sources, utm_id: action.payload } }
            break;
        case 'utmTermChanged':
            newState = { ...state, sources: { ...state.sources, utm_term: action.payload } }
            break;
        case 'utmContentChanged':
            newState = { ...state, sources: { ...state.sources, utm_content: action.payload } }
            break
        case 'warningChanged':
            newState = { ...state, sources: { ...state.sources, warning: action.payload } }
            break
        case 'onFetching':
            newState = { ...state, sources: { ...state.sources, isFetching: true } }
            break
        case 'sources:onFetched':
            let { items, total, segment } = action.payload;
            newState = {
                ...state,
                sources: {
                    ...state.sources,
                    isFetching: false,
                    items,
                    total,
                    total_segment: segment,
                }
            }
            break

        case 'courses.onFetching':
            newState = { ...state, courses: { ...state.courses, isFetching: true, warning: '', error: '', } }
            break;
        case 'courses.onFetched':
            newState = {
                ...state,
                courses: {
                    ...state.courses,
                    items: action.payload.items,
                    is_segment: action.payload.is_segment
                }
            }
            break;
        case 'courses.onEnd':
            newState = { ...state, courses: { ...state.courses, isFetching: false, } }
            break;
        case 'courses.levelsChanged':
            newState = { ...state, courses: { ...state.courses, levels: action.payload, warning: '', error: '', } }
            break;
        case 'courses.dateFromChanged':
            newState = { ...state, courses: { ...state.courses, dateFrom: action.payload, error: '', } }
            break;
        case 'courses.dateToChanged':
            newState = { ...state, courses: { ...state.courses, dateTo: action.payload, error: '', } }
            break;
        case 'courses.courseChanged':
            newState = {
                ...state,
                courses: { ...state.courses, course: action.payload, levels: ['all'], warning: '', error: '', }
            }
            break;
        case 'courses.switchedSegmentTab':
            newState = {
                ...state,
                courses: { ...state.courses, segment_tab: action.payload }
            }
            break;
        case 'courses.segmentChange':
            newState = { ...state, courses: { ...state.courses, segment: action.payload, warning: '', } }
            break
        case 'courses.onWarning':
            newState = { ...state, courses: { ...state.courses, warning: action.payload } }
            break
        case 'courses.onError':
            newState = { ...state, courses: { ...state.courses, error: action.payload } }
            break

        case 'sales.warningChanged':
            newState = { ...state, sales: { ...state.sales, warning: action.payload } }
            break;
        case 'sales.onFetching':
            newState = { ...state, sales: { ...state.sales, warning: '', isFetching: true } }
            break;
        case 'sales.onFetched':
            let { sale, no_sale, sale_segment, no_sale_segment } = action.payload.data
            newState = {
                ...state,
                sales: {
                    ...state.sales,
                    isFetching: false,
                    sale,
                    no_sale,
                    sale_segment,
                    no_sale_segment,
                    is_segment: action.payload.is_segment
                }
            }
            break;
        case 'sales.levelsChanged':
            let { val, all_levels } = action.payload
            if (val.includes(state.sales.level)) {
                newState = { ...state, sales: { ...state.sales, levels: val, level: '', warning: '', } }
            } else {
                newState = { ...state, sales: { ...state.sales, levels: val, warning: '', } }
            }
            break;
        case 'sales.dateFromChanged':
            newState = { ...state, sales: { ...state.sales, dateFrom: action.payload } }
            break;
        case 'sales.dateToChanged':
            newState = { ...state, sales: { ...state.sales, dateTo: action.payload } }
            break;
        case 'sales.levelChanged':
            newState = { ...state, sales: { ...state.sales, level: action.payload, warning: '', } }
            break;
        case 'sales.switchedSegmentTab':
            newState = {
                ...state,
                sales: { ...state.sales, segment_tab: action.payload }
            }
            break;
        case 'sales.segmentChange':
            newState = { ...state, sales: { ...state.sales, segment: action.payload, warning: '', } }
            break

        case 'expiring.warningChanged':
            newState = { ...state, expiring: { ...state.expiring, warning: action.payload } }
            break;
        case 'expiring.itemsFetching':
            newState = { ...state, expiring: { ...state.expiring, disabled: true } }
            break;
        case 'expiring.itemsFetched':
            newState = {
                ...state,
                expiring: {
                    ...state.expiring,
                    items: action.payload.items,
                    total_segment: action.payload.total_segment,
                    is_segment: action.payload.is_segment,
                    disabled: false
                }
            }
            break;
        case 'expiring.dateFromChanged':
            if (action.payload > state.expiring.dateTo) {
                newState = {
                    ...state,
                    expiring: {
                        ...state.expiring,
                        dateFrom: action.payload,
                        dateTo: action.payload,
                        warning: ''
                    }
                }
            } else {
                newState = { ...state, expiring: { ...state.expiring, dateFrom: action.payload, warning: '' } }
            }
            break;
        case 'expiring.dateToChanged':
            if (action.payload < state.expiring.dateFrom) {
                newState = {
                    ...state,
                    expiring: {
                        ...state.expiring,
                        dateFrom: action.payload,
                        dateTo: action.payload,
                        warning: ''
                    }
                }
            } else {
                newState = { ...state, expiring: { ...state.expiring, dateTo: action.payload, warning: '' } }
            }
            break;
        case 'expiring.levelChanged':
            newState = { ...state, expiring: { ...state.expiring, level: action.payload, warning: '' } }
            break;
        case 'expiring.switchedSegmentTab':
            newState = {
                ...state,
                expiring: { ...state.expiring, segment_tab: action.payload }
            }
            break;
        case 'expiring.segmentChange':
            newState = { ...state, expiring: { ...state.expiring, segment: action.payload, warning: '', } }
            break

        case 'onClickSegment':
            newState = { ...state, segment: { ...action.payload, name: '', description: '' }, segmentPopup: true }
            break;
        case 'closeSegmentPopup':
            newState = { ...state, segmentPopup: false, segmentSuccess: false, segmentError: '' }
            break;
        case 'segment:saving':
            newState = { ...state, segmentError: '', segmentSaving: true, segmentSuccess: false }
            break;
        case 'segment:complete':
            newState = { ...state, segmentSaving: false }
            break;
        case 'segment:saved':
            let { segments } = action.payload
            newState = { ...state, segmentSuccess: true, segmentPopup: false, ssr: { ...state.ssr, segments } }
            break;
        case 'segment:changeName':
            newState = {
                ...state,
                segment: { ...state.segment, name: action.payload },
                nameInvalid: action.payload.length === 0,
            }
            break;
        case 'segment:changeDescription':
            newState = {
                ...state,
                segment: { ...state.segment, description: action.payload },
                descriptionInvalid: action.payload.length === 0,
            }
            break;
        case 'segment:error':
            newState = { ...state, segmentError: action.payload }
            break;
        case 'segment:validation':
            newState = {
                ...state,
                nameInvalid: state.segment.name.length === 0,
                descriptionInvalid: state.segment.description.length === 0,
            }
            break;
        case 'segment:closeSuccess':
            newState = { ...state, segmentSuccess: false }
            break;

        case 'segmentTable.blocked':
            newState = { ...state, segmentTable: { ...state.segmentTable, blocking: action.payload } }
            break;
        case 'segmentTable.completed':
            newState = { ...state, segmentTable: { ...state.segmentTable, completed: true } }
            break;
        case 'segmentTable.failed':
            newState = { ...state, segmentTable: { ...state.segmentTable, error: action.payload } }
            break;
        case 'segmentTable.successNoticeClosed':
            newState = { ...state, segmentTable: { ...state.segmentTable, completed: false } }
            break;
        case 'segmentTable.remove':
            newState = { ...state, ssr: {
                ...state.ssr,
                    segments: state.ssr.segments.filter(e => e.id !== action.payload)
            } }
            break;

        default:
            newState = { ...state }
    }

    return newState;
}
