<?php

namespace Mbl\Analytics;

use WP_Post;
use WP_Query;
use WP_Term;

class Course
{
    private int $term_id;
    private $lessons = null;
    private $levels = null;

    /**
     * @param int $term_id
     */
    public function __construct(int $term_id)
    {
        $this->term_id = $term_id;
    }

    /**
     * @return WP_Post[]
     */
    public function lessons(): array
    {
        if (is_null($this->lessons)) {
            $this->lessons = (new WP_Query([
                'post_type' => 'wpm-page',
                'posts_per_page' => -1,
                'post_status' => 'publish',
                'tax_query' => array(
                    array(
                        'taxonomy' => 'wpm-category',
                        'field' => 'term_id',
                        'terms' => $this->term_id
                    )
                ),
                'orderby' => 'menu_order title',
                'order' => 'ASC'
            ]))->posts;
        }
        return $this->lessons ?? [];
    }

    /**
     * @return WP_Term[]
     */
    public function levels(): array
    {
        if (is_null($this->levels)) {
            $byId = [];
            foreach ($this->lessons() as $post) {
                $postAccessLevels = wp_get_post_terms($post->ID, 'wpm-levels');
                foreach ($postAccessLevels as $term) {
                    if (isset($byId[$term->term_id])) {
                        continue;
                    }
                    $byId[$term->term_id] = $term;
                }
            }
            $this->levels = array_values($byId);
        }

        return $this->levels;
    }

    public function is_autotraining(): bool {
        return wpm_is_autotraining($this->term_id);
    }
}
