<?php

namespace Mbl\Analytics;

use DateTime;

class CourseStatUserSelectionByFilter implements CourseStatUserSelection
{
    private array $levels;
    private DateTime $dateFrom;
    private DateTime $dateTo;

    /**
     * @param array $levels
     * @param DateTime $dateFrom
     * @param DateTime $dateTo
     */
    public function __construct(array $levels, DateTime $dateFrom, DateTime $dateTo)
    {
        $this->levels = $levels;
        $this->dateFrom = $dateFrom;
        $this->dateTo = $dateTo;
    }


    public function get_ids(Course $course): array
    {
        global $wpdb;
        if ($this->levels[0] === 'all') {
            $term_ids = array_map(fn($term) => $term->term_id, $course->levels());
        } else {
            $term_ids = $this->levels;
        }
        $levels = array_map(fn($term_id) => (int)$term_id, $term_ids);
        /**
         * Caching
         */
        get_terms([
            'taxonomy' => 'wpm-levels',
            'include' => $levels
        ]);
        $levels_in = implode(', ', $levels);
        $levels_where = "tk.term_id IN ($levels_in)";
        $dur_where = sprintf(
            "(
                        (tk.date_end >= '%1\$s' AND tk.date_start <= '%2\$s' AND tk.is_unlimited != 1)
                        OR (tk.date_start <= '%2\$s' AND tk.is_unlimited = 1)
                    )",
            $this->dateFrom->format('Y-m-d'), $this->dateTo->format('Y-m-d')
        );
        $sql = "SELECT DISTINCT tk.user_id
        FROM `{$wpdb->prefix}memberlux_term_keys` AS tk
        JOIN `{$wpdb->prefix}usermeta` AS um ON um.user_id = tk.user_id AND  um.meta_key = '{$wpdb->prefix}capabilities'
        WHERE $levels_where AND tk.is_banned != 1 AND tk.key_type = 'wpm_term_keys' AND $dur_where
            AND um.meta_value LIKE '%customer%'
        ";

        return $wpdb->get_col($sql);
    }

}
