<?php

namespace Mbl\Analytics;

class  CourseUsers
{
    private Course $course;

    /**
     * @param Course $course
     */
    public function __construct( Course $course )
    {
        $this->course = $course;
    }

    public function toArray( UserSelection $selection, $range ): array
    {
        $users = [];

        global $wpdb;
        $allLessons = $this->course->lessons();
        $posts_id = array_map( fn( $post ) => $post->ID, $allLessons );
        $users_id = array_map( 'absint', $wpdb->get_col( $selection->idsRequest() ) );
        $users_id_in = "user_id IN({$selection->idsRequest()})";
        $posts_id_in = 'post_id IN (' . implode( ', ', $posts_id ) . ')';

        $views = array_reduce(
            $wpdb->get_results(
                "SELECT user_id, post_id" . PHP_EOL
                . "FROM {$wpdb->prefix}memberlux_lesson_view" . PHP_EOL
                . "WHERE $users_id_in AND $posts_id_in",
                ARRAY_A
            ),
            function ( $carry, $item ) {
                $carry["{$item['post_id']}_{$item['user_id']}"] = 1;
                return $carry;
            },
            []
        );

        foreach ( $users_id as $user_id ) {
            $levels = all_user_accessible_levels(
                $user_id,
                $range->from()->format('Y-m-d'),
                $range->to()->format('Y-m-d')
            );

            $lessons = [];
            $prevPassed = true;
            foreach ( $allLessons as $index => $post ) {
                $postMeta = get_post_meta( $post->ID, '_wpm_page_meta', true );
                $postAccessLevels = wp_get_post_terms( $post->ID, 'wpm-levels' );
                $hasHomework = wpm_array_get( $postMeta, 'is_homework' ) == 'on';

                $isAccess = empty( $postAccessLevels ) || !empty( array_intersect(
                        array_map( fn( $i ) => $i->term_id, $postAccessLevels ),
                        $levels
                    ) );

                if ( $isAccess ) {
                    $isVisible = true;
                } else {
                    $isVisible = false;
                    foreach ( $postAccessLevels as $level ) {
                        $termId = $level->term_id;
                        $termMeta = get_option( "taxonomy_term_$termId" );
                        if ( $termMeta['hide_for_no_access'] != 'hide' ) {
                            $isVisible = true;
                            break;
                        }
                    }
                }

                $responseStatus = null;
                $isPassed = false;
                if ( $hasHomework ) {
                    $homework = new Homework( $post, $user_id );
                    $responseStatus = $homework->getResponseStatus();
                    $isPassed = $isVisible && $homework->isCompleted();
                } else {
                    $passed = [];
                    if ( $isVisible ) {
                        $passed = get_user_meta( $user_id, 'passed_materials', true );
                        $passed = is_array( $passed ) ? $passed : [];
                    }
                    $isPassed = wpm_array_get( $passed, $post->ID, false );
                }

                if ($this->course->is_autotraining()) {
                    $isPassed = $prevPassed && $isPassed;
                }

                $lessons[] = [
                    'index' => $index,
                    'passed' => $isPassed,
                    'prevPassed' => $prevPassed,
                    'isAccess' => $isAccess,
                    'isVisible' => $isVisible,
                    'opened' => isset( $views["{$post->ID}_$user_id"] ),
                    'responseStatus' => $responseStatus,
                ];

                $prevPassed = $isPassed;
            }

            $users[] = [
                'user_id' => $user_id,
                'lessons' => $lessons
            ];
        }

        return $users;
    }
}
