<?php

namespace Mbl\Analytics;

class CoursesStat
{
    private Course $course;

    /**
     * @param Course $course
     */
    public function __construct( Course $course )
    {
        $this->course = $course;
    }

    public function toArray( UserSelection $selection, DateRange $range): array
    {
        $result = [];
        $users = (new CourseUsers( $this->course ))->toArray( $selection, $range );
        $allLessons = $this->course->lessons();

        foreach ( $users as $user ) {
            foreach ( $user['lessons'] as $index => $lesson ) {
                $post = $allLessons[$index];
                $postMeta = get_post_meta( $post->ID, '_wpm_page_meta', true );
                $hasHomework = wpm_array_get( $postMeta, 'is_homework' ) == 'on';

                if ( !isset( $result[$post->ID] ) ) {
                    $result[$post->ID] = [
                        'id' => $post->ID,
                        'name' => $post->post_title,
                        'available' => 0,
                        'opened' => 0,
                        'no_opened' => 0,
                        'homework' => [
                            'sent' => $hasHomework ? 0 : '-',
                            'no_sent' => $hasHomework ? 0 : '-',
                            'opened' => $hasHomework ? 0 : '-',
                            'approved' => $hasHomework ? 0 : '-',
                            'rejected' => $hasHomework ? 0 : '-',
                        ]
                    ];
                }

                $lessonData = new LessonData( $lesson, $this->course );

                if ( $lessonData->isAvailable() ) {
                    $result[$post->ID]['available'] += 1;
                    $result[$post->ID][$lessonData->isOpened() ? 'opened' : 'no_opened'] += 1;
                }

                if ( !$hasHomework || !$lessonData->isVisible() || !$lessonData->isAvailable() ) {
                    continue;
                }

                if ( $lessonData->homeworkIsOpened() ) {
                    $result[$post->ID]['homework']['opened'] += 1;
                }

                if ( $lessonData->homeworkIsApproved() ) {
                    $result[$post->ID]['homework']['approved'] += 1;
                }

                if ( $lessonData->homeworkIsRejected() ) {
                    $result[$post->ID]['homework']['rejected'] += 1;
                }

                if ( $lessonData->homeworkIsSent() ) {
                    $result[$post->ID]['homework']['sent'] += 1;
                } else {
                    if ( $this->course->is_autotraining() ) {
                        if ( $index === 0 || ($index > 0 && $lessonData->prevIsPassed()) ) {
                            $result[$post->ID]['homework']['no_sent'] += 1;
                        }
                    } else {
                        $result[$post->ID]['homework']['no_sent'] += 1;
                    }
                }
            }
        }

        return array_values( $result );
    }
}
