<?php

namespace Mbl\Analytics;

use Mbl\Analytics\Filter\CnRoleCustomer;
use Mbl\Analytics\Filter\CnUtmCampaign;
use Mbl\Analytics\Filter\CnUtmContent;
use Mbl\Analytics\Filter\CnUtmId;
use Mbl\Analytics\Filter\CnUtmMedium;
use Mbl\Analytics\Filter\CnUtmSource;
use Mbl\Analytics\Filter\CnUtmTerm;
use Mbl\Analytics\Filter\CnWithTable;
use Mbl\Analytics\Filter\CnAccess;
use Mbl\Analytics\Filter\CnDateStartBetween;
use Mbl\Analytics\Filter\CnDurationType;
use Mbl\Analytics\Filter\CnNotNullUser;
use Mbl\Analytics\Filter\SimpleSelectorFilters;
use Mbl\Analytics\Filter\SmTable;
use Mbl\Analytics\Filter\CnSource;
use Mbl\Analytics\Filter\STermKeysFrom;
use Mbl\Analytics\Filter\TermKeysFromWithMeta;
use Mbl\Analytics\Filter\TermKeysFromWithUser;
use Mbl\Analytics\Filter\CnValidAccess;

class SmSourceStRawData implements SourcesStatisticsRawData
{
    private array $params;

    /**
     * @param array $params
     */
    public function __construct( array $params )
    {
        $this->params = $params;
    }


    public function toArray(): array
    {
        global $wpdb;
        $keysTable = new SmTable( 'tk' );
        $keysMetaTable = new SmTable( 'km' );
        $userMetaTable = new SmTable( 'um' );

        $filter = new SimpleSelectorFilters( [
            new CnWithTable( new CnDurationType( $this->params['duration'] ), $keysTable ),
            new CnWithTable(
                new CnDateStartBetween( new SDateRange( $this->params['start'], $this->params['end'] ) ), $keysTable
            ),
            new CnWithTable( new CnUtmSource( $this->params['utm_source'] ), $keysMetaTable ),
            new CnWithTable( new CnUtmMedium( $this->params['utm_medium'] ), $keysMetaTable ),
            new CnWithTable( new CnUtmCampaign( $this->params['utm_campaign'] ), $keysMetaTable ),
            new CnWithTable( new CnUtmContent( $this->params['utm_content'] ), $keysMetaTable ),
            new CnWithTable( new CnUtmTerm( $this->params['utm_term'] ), $keysMetaTable ),
            new CnWithTable( new CnUtmId( $this->params['utm_id'] ), $keysMetaTable ),
            new CnWithTable( new CnSource( $this->params['source'] ), $keysMetaTable ),
            new CnWithTable( new CnAccess( $this->params['terms'] ), $keysTable ),
            new CnWithTable( new CnValidAccess(), $keysTable ),
            new CnWithTable( new CnNotNullUser(), $keysTable ),
            new CnWithTable( new CnRoleCustomer(), $userMetaTable )
        ] );

        $from = new TermKeysFromWithMeta(
            new TermKeysFromWithUser(
                new STermKeysFrom( $keysTable ), $userMetaTable
            ), $keysMetaTable
        );

        $request = "{$from->clause()}
        WHERE {$filter->toWhere()}";

        $sqlItems = $keysMetaTable->withAlias( $keysTable->withAlias(
            "SELECT COUNT(tk.user_id) AS count, tk.term_id term_id,
            tk.duration duration, tk.units unit, km.source, tk.is_unlimited,
            km.utm_source, km.utm_medium, km.utm_campaign, km.utm_id, km.utm_term, km.utm_content 
            $request
            GROUP BY tk.term_id, km.source, tk.is_unlimited, tk.units, tk.duration,
                km.utm_source, km.utm_medium, km.utm_campaign, km.utm_id, km.utm_term, km.utm_content
            ORDER BY tk.term_id, km.source, tk.is_unlimited, tk.units, tk.duration,
                km.utm_source, km.utm_medium, km.utm_campaign, km.utm_id, km.utm_term, km.utm_content"
            , 'tk'
        ), 'km' );
        $sqlTotal = "SELECT COUNT(DISTINCT tk.user_id) $request";
        $total = absint( $wpdb->get_var( $sqlTotal ) );
        $items = [];

        if ( $total > 0 ) {
            $items = $wpdb->get_results( $sqlItems, ARRAY_A );
        }

        return [ 'total' => $total, 'items' => $items ];
    }
}
