<?php

namespace Mbl\Analytics;

use DateTime;
use Exception;

class SmSourcesSt implements SourcesStatistics
{
    private SourcesStatisticsRawData $rawData;

    /**
     * @param SourcesStatisticsRawData $rawData
     */
    public function __construct( SourcesStatisticsRawData $rawData )
    {
        $this->rawData = $rawData;
    }

    /**
     * @throws Exception
     */
    public function toArray(): array
    {
        $result = [];

        $sourceStData = $this->rawData->toArray();
        $all_sources = mbl_access_level_sources();
        /**
         * Caching
         */
        get_terms( [
            'taxonomy' => 'wpm-levels',
            'hide_empty' => false
        ] );

        foreach ( $sourceStData['items'] as $item ) {
            $term = get_term( $item['term_id'], 'wpm-levels' );
            $source_id = $item['source'];
            if ( !isset( $result[$item['term_id']] ) ) {
                $result[$item['term_id']] = [
                    'term_id' => $term->term_id,
                    'name' => $term->name,
                    'items' => []
                ];
            }
            $is_unlimited = (int)$item['is_unlimited'];

            if ( $is_unlimited === 1 ) {
                $duration_d = 'Неограниченный';
            } else if ( $item['unit'] === 'days' ) {
                $duration_d = sprintf( "%s дн.", $item['duration'] );
            } else {
                $duration_d = sprintf( "%s мес.", $item['duration'] );
            }
            $result[$term->term_id]['items'][] = [
                'source' => [
                    'id' => $source_id,
                    'name' => $all_sources[$source_id]
                ],
                'is_unlimited' => $is_unlimited,
                'duration' => $item['duration'],
                'unit' => $item['unit'],
                'utm_source' => $item['utm_source'],
                'utm_medium' => $item['utm_medium'],
                'utm_campaign' => $item['utm_campaign'],
                'utm_id' => $item['utm_id'],
                'utm_term' => $item['utm_term'],
                'utm_content' => $item['utm_content'],
                'duration_d' => $duration_d,
                'count' => (int)$item['count']
            ];
        }

        return [
            'items' => array_values( $result ),
            'total' => $sourceStData['total'],
        ];
    }

    /**
     * @throws Exception
     */
    public function usersRequest( $levels, $source, $start, $end, $duration, $units = null ): string
    {
        global $wpdb;
        $startDate = new DateTime( $start );
        $endDate = new DateTime( $end );
        if ( $levels[0] === 'all' ) {
            $levels_where = '1';
        } else {
            $levels = array_map( fn( $term_id ) => (int)$term_id, $levels );
            $levels_in = implode( ', ', $levels );
            $levels_where = "tk.term_id IN ($levels_in)";
        }
        $duration_where = "1";
        if ( in_array( $duration, [ 'all', 'limited', 'unlimited' ] ) ) {
            if ( $duration === 'limited' ) {
                $duration_where = "tk.is_unlimited != 1";
            } elseif ( $duration === 'unlimited' ) {
                $duration_where = "tk.is_unlimited = 1";
            }
        } else {
            if ( is_null( $units ) ) {
                throw new Exception( 'The units parameter must not be null' );
            }
            $duration_where = "tk.duration = '$duration' AND tk.units = '$units'";
        }
        $date_start_where = sprintf(
            "tk.date_start BETWEEN '%s' AND '%s'",
            $startDate->format( 'Y-m-d' ), $endDate->format( 'Y-m-d' )
        );
        if ( $source[0] === 'all' ) {
            $sources_where = "km.source IS NOT NULL AND km.source != ''";
        } else {
            $sources = implode( ', ', array_map( fn( $s ) => "'$s'", $source ) );
            $sources_where = "km.source IN ($sources)";
        }
        return "FROM `{$wpdb->prefix}memberlux_term_keys` AS tk
            JOIN `{$wpdb->prefix}memberlux_keys_meta` AS km ON km.key_id = tk.id
            JOIN `{$wpdb->prefix}usermeta` AS um 
                ON tk.user_id = um.user_id AND  um.meta_key = '{$wpdb->prefix}capabilities' 
            WHERE tk.is_banned != 1 AND tk.key_type = 'wpm_term_keys' AND um.meta_value LIKE '%customer%'
                AND $levels_where AND $date_start_where AND $duration_where AND $sources_where";
    }
}
