<?php

namespace Mbl\Analytics;

use Exception;

class SourcesStatSegmented
{
    private array $params;

    /**
     * @param array $params
     */
    public function __construct( array $params )
    {
        $this->params = $params;
    }

    /**
     * @throws Exception
     */
    public function toArray(): array
    {
        $data = (new SmSourcesSt( new SmSourceStRawData( $this->params ) ))->toArray();
        $startDate = date( 'Y-m-d', strtotime( $this->params['start'] ) );
        $endDate = date( 'Y-m-d', strtotime( $this->params['end'] ) );
        $data['segment'] = [
            'type' => 'sources',
            'params' => [
                'from' => $startDate,
                'to' => $endDate,
                'levels' => $this->params['terms'],
                'duration' => $this->params['duration'],
                'source' => $this->params['source'],
                'utm_source' => $this->params['utm_source'],
                'utm_medium' => $this->params['utm_medium'],
                'utm_campaign' => $this->params['utm_campaign'],
                'utm_id' => $this->params['utm_id'],
                'utm_term' => $this->params['utm_term'],
                'utm_content' => $this->params['utm_content']
            ]
        ];
        foreach ( $data['items'] as $key => $group ) {
            foreach ( $group['items'] as $index => $item ) {
                if ( $item['is_unlimited'] === 1 ) {
                    $durationData = [
                        'is_unlimited' => true,
                    ];
                } else {
                    $durationData = [
                        'is_unlimited' => false,
                        'duration' => $item['duration'],
                        'units' => $item['unit']
                    ];
                }
                $data['items'][$key]['items'][$index]['segment'] = [
                    'type' => 'sources',
                    'params' => [
                            'from' => $startDate,
                            'to' => $endDate,
                            'levels' => [ $group['term_id'] ],
                            'source' => [ $item['source']['id'] ],
                            'utm_source' => $item['utm_source'],
                            'utm_medium' => $item['utm_medium'],
                            'utm_campaign' => $item['utm_campaign'],
                            'utm_id' => $item['utm_id'],
                            'utm_term' => $item['utm_term'],
                            'utm_content' => $item['utm_content']
                        ] + $durationData
                ];
            }
        }

        return $data;
    }
}
