<?php

namespace Mbl\Analytics;

use Exception;
use Mbl\Analytics\Filter\CnAccess;
use Mbl\Analytics\Filter\CnDateStartBetween;
use Mbl\Analytics\Filter\CnDurationType;
use Mbl\Analytics\Filter\CnNotNullUser;
use Mbl\Analytics\Filter\CnRoleCustomer;
use Mbl\Analytics\Filter\CnSource;
use Mbl\Analytics\Filter\CnUtmCampaign;
use Mbl\Analytics\Filter\CnUtmContent;
use Mbl\Analytics\Filter\CnUtmId;
use Mbl\Analytics\Filter\CnUtmMedium;
use Mbl\Analytics\Filter\CnUtmSource;
use Mbl\Analytics\Filter\CnUtmTerm;
use Mbl\Analytics\Filter\CnValidAccess;
use Mbl\Analytics\Filter\CnWithTable;
use Mbl\Analytics\Filter\SimpleSelectorFilters;
use Mbl\Analytics\Filter\SmTable;
use Mbl\Analytics\Filter\STermKeysFrom;
use Mbl\Analytics\Filter\TermKeysFromWithMeta;
use Mbl\Analytics\Filter\TermKeysFromWithUser;

class UserBySourcesByDuration implements UserSelection
{
    private DateRange $range;
    private array $levels;
    /**
     * @var string[]
     */
    private array $sources;
    /**
     * @var string 'all', 'limited', 'unlimited'
     */
    private $duration;

    private $units = null;
    private string $utm_source;
    private string $utm_medium;
    private string $utm_campaign;
    private string $utm_content;
    private string $utm_term;
    private string $utm_id;

    /**
     * @param int[] $levels
     * @param string[] $sources
     * @param DateRange $range
     * @param string | int $duration
     * @param string | null $units
     * @param string $utm_source
     * @param string $utm_medium
     * @param string $utm_campaign
     * @param string $utm_content
     * @param string $utm_term
     * @param string $utm_id
     */
    public function __construct(
        array     $levels,
        array     $sources,
        DateRange $range,
                  $duration,
        string    $units = null,
        string    $utm_source = '',
        string    $utm_medium = '',
        string    $utm_campaign = '',
        string    $utm_content = '',
        string    $utm_term = '',
        string    $utm_id = ''
    )
    {
        $this->range = $range;
        $this->levels = $levels;
        $this->sources = $sources;
        $this->duration = $duration;
        $this->units = $units;
        $this->utm_source = $utm_source;
        $this->utm_medium = $utm_medium;
        $this->utm_campaign = $utm_campaign;
        $this->utm_content = $utm_content;
        $this->utm_term = $utm_term;
        $this->utm_id = $utm_id;
    }

    /**
     * @throws Exception
     */
    public function request( string $tablePrefix = '' ): string
    {

        $keysTable = new SmTable( "tk$tablePrefix" );
        $keysMetaTable = new SmTable( "km$tablePrefix" );
        $userMetaTable = new SmTable( "um$tablePrefix" );

        $filter = new SimpleSelectorFilters( [
            new CnWithTable( new CnDurationType( $this->duration, $this->units ), $keysTable ),
            new CnWithTable( new CnDateStartBetween( $this->range ), $keysTable ),
            new CnWithTable( new CnSource( $this->sources ), $keysMetaTable ),
            new CnWithTable( new CnUtmSource( $this->utm_source ), $keysMetaTable ),
            new CnWithTable( new CnUtmMedium( $this->utm_medium ), $keysMetaTable ),
            new CnWithTable( new CnUtmCampaign( $this->utm_campaign ), $keysMetaTable ),
            new CnWithTable( new CnUtmContent( $this->utm_content ), $keysMetaTable ),
            new CnWithTable( new CnUtmTerm( $this->utm_term ), $keysMetaTable ),
            new CnWithTable( new CnUtmId( $this->utm_id ), $keysMetaTable ),
            new CnWithTable( new CnAccess( $this->levels ), $keysTable ),
            new CnWithTable( new CnValidAccess(), $keysTable ),
            new CnWithTable( new CnNotNullUser(), $keysTable ),
            new CnWithTable( new CnRoleCustomer(), $userMetaTable )
        ] );

        $from = new TermKeysFromWithMeta(
            new TermKeysFromWithUser(
                new STermKeysFrom( $keysTable ), $userMetaTable
            ), $keysMetaTable
        );

        return $from->clause() . PHP_EOL . "WHERE {$filter->toWhere()}";
    }

    /**
     * @throws Exception
     */
    public function idsRequest( string $tablePrefix = '' ): string
    {
        $keysTb = "tk$tablePrefix";
        return "SELECT DISTINCT $keysTb.user_id {$this->request($tablePrefix)}";
    }
}
