<?php

namespace Mbl\Analytics;

use Mbl\Analytics\Filter\CnDateEndBetween;
use Mbl\Analytics\Filter\CnDurationType;
use Mbl\Analytics\Filter\CnNotNullUser;
use Mbl\Analytics\Filter\CnRoleCustomer;
use Mbl\Analytics\Filter\CnSpecifiedAccess;
use Mbl\Analytics\Filter\CnValidAccess;
use Mbl\Analytics\Filter\CnWithTable;
use Mbl\Analytics\Filter\SimpleSelectorFilters;
use Mbl\Analytics\Filter\SmTable;
use Mbl\Analytics\Filter\STermKeysFrom;
use Mbl\Analytics\Filter\TermKeysFromWithUser;

class UserExpiringAccess implements UserSelection
{
    private int $level_id;
    private DateRange $range;
    /**
     * @var string|int
     */
    private $duration;
    /**
     * @var string|null
     */
    private ?string $units;

    /**
     * @param int $level_id
     * @param DateRange $range
     * @param string|int $duration
     * @param null $units
     */
    public function __construct( int $level_id, DateRange $range, $duration = 'limited', $units = null )
    {
        $this->level_id = $level_id;
        $this->range = $range;
        $this->duration = $duration;
        $this->units = $units;
    }

    public function request( string $tablePrefix = '' ): string
    {
        $keysTable = new SmTable( "tk$tablePrefix" );
        $userMetaTable = new SmTable( "um$tablePrefix" );
        $filter = new SimpleSelectorFilters( [
            new CnWithTable( new CnDurationType( $this->duration, $this->units ), $keysTable ),
            new CnWithTable( new CnSpecifiedAccess( $this->level_id ), $keysTable ),
            new CnWithTable( new CnDateEndBetween( $this->range ), $keysTable ),
            new CnWithTable( new CnValidAccess(), $keysTable ),
            new CnWithTable( new CnRoleCustomer(), $userMetaTable ),
            new CnWithTable( new CnNotNullUser(), $keysTable ),
        ] );
        $from = new TermKeysFromWithUser( new STermKeysFrom( $keysTable ), $userMetaTable );

        return $from->clause() . PHP_EOL . "WHERE " . $filter->toWhere();
    }

    public function idsRequest( string $tablePrefix = '' ): string
    {
        $keysTb = "tk$tablePrefix";

        return "SELECT DISTINCT $keysTb.user_id {$this->request($tablePrefix)}";
    }
}
