<?php

namespace Mbl\Analytics;

use Mbl\Analytics\Filter\CnDateStartBetween;
use Mbl\Analytics\Filter\CnNotNullUser;
use Mbl\Analytics\Filter\CnRoleCustomer;
use Mbl\Analytics\Filter\CnValidAccess;
use Mbl\Analytics\Filter\CnWithTable;
use Mbl\Analytics\Filter\SimpleSelectorFilters;
use Mbl\Analytics\Filter\SmTable;
use Mbl\Analytics\Filter\CnSpecifiedAccesses;
use Mbl\Analytics\Filter\STermKeysFrom;
use Mbl\Analytics\Filter\TermKeysFromWithUser;

class UserGotAccessDuringPeriod implements UserSelection {
	private array $levels;
	private DateRange $range;

	/**
	 * @param int[] $levels
	 * @param DateRange $range
	 */
	public function __construct( array $levels, DateRange $range ) {
		$this->levels = $levels;
		$this->range  = $range;
	}

	public function request( string $tablePrefix = '' ): string {
		$keysTable     = new SmTable( "tk$tablePrefix" );
		$userMetaTable = new SmTable( "um$tablePrefix" );
		$from          = new TermKeysFromWithUser( new STermKeysFrom( $keysTable ), $userMetaTable );
		$filter        = new SimpleSelectorFilters( [
			new CnWithTable( new CnSpecifiedAccesses( $this->levels ), $keysTable ),
			new CnWithTable( new CnDateStartBetween( $this->range ), $keysTable ),
			new CnWithTable( new CnValidAccess(), $keysTable ),
			new CnWithTable( new CnRoleCustomer(), $userMetaTable ),
			new CnWithTable( new CnNotNullUser(), $keysTable ),
		] );

		return $from->clause() . PHP_EOL . "WHERE " . $filter->toWhere();
	}

	public function idsRequest( string $tablePrefix = '' ): string {
		$keysTb = "tk$tablePrefix";

		return "SELECT DISTINCT $keysTb.user_id {$this->request($tablePrefix)}";
	}
}
