<?php

namespace Mbl\Analytics;

use Mbl\Analytics\Filter\CnDateStartBetween;
use Mbl\Analytics\Filter\CnSpecifiedAccess;
use Mbl\Analytics\Filter\CnWithTable;
use Mbl\Analytics\Filter\SimpleSelectorFilters;
use Mbl\Analytics\Filter\SmTable;

class UserHasAccessNotPurchased implements UserSelection {
	private int $levelId;
	/**
	 * @var int[]
	 */
	private array $levels;
	private DateRange $range;

	/**
	 * @param int $levelId
	 * @param int[] $levels
	 * @param DateRange $range
	 */
	public function __construct( int $levelId, array $levels, DateRange $range ) {
		$this->levelId = $levelId;
		$this->levels  = $levels;
		$this->range   = $range;
	}

	public function request( string $tablePrefix = '' ): string {
		global $wpdb;
		$keysTable = new SmTable( "tk_s$tablePrefix" );
		$filter    = new SimpleSelectorFilters( [
			new CnWithTable( new CnSpecifiedAccess( $this->levelId ), $keysTable ),
			new CnWithTable( new CnDateStartBetween( $this->range ), $keysTable ),
		] );
		$selection = new UserGotAccessDuringPeriod( $this->levels, $this->range );

		return "FROM `{$wpdb->prefix}memberlux_term_keys` AS tk$tablePrefix
        WHERE tk$tablePrefix.user_id IN({$selection->idsRequest("ga$tablePrefix")}) 
        AND tk$tablePrefix.user_id NOT IN (
            SELECT DISTINCT tk_s$tablePrefix.user_id
            FROM `{$wpdb->prefix}memberlux_term_keys` AS tk_s$tablePrefix
            WHERE {$filter->toWhere()}
        )";
	}

	public function idsRequest( string $tablePrefix = '' ): string {
		return "SELECT DISTINCT tk$tablePrefix.user_id {$this->request($tablePrefix)}";
	}
}
