<?php

namespace Mbl\Analytics;

use Mbl\Analytics\Filter\CnDateStartBetween;
use Mbl\Analytics\Filter\CnSpecifiedAccess;
use Mbl\Analytics\Filter\CnWithTable;
use Mbl\Analytics\Filter\SimpleSelectorFilters;
use Mbl\Analytics\Filter\SmTable;
use Mbl\Analytics\Filter\STermKeysFrom;
use Mbl\Analytics\Filter\TermKeysFromWithUser;

class UserHasAccessPurchased implements UserSelection
{
    private int $levelId;
    /**
     * @var int[]
     */
    private array $levels;
    private DateRange $range;

    /**
     * @param int $levelId
     * @param int[] $levels
     * @param DateRange $range
     */
    public function __construct(int $levelId, array $levels, DateRange $range)
    {
        $this->levelId = $levelId;
        $this->levels = $levels;
        $this->range = $range;
    }

    public function request(string $tablePrefix = ''): string
    {
        $keysTable = new SmTable("tk$tablePrefix");
        $userMetaTable = new SmTable("um$tablePrefix");
        $filter = new SimpleSelectorFilters([
            new CnWithTable(new CnSpecifiedAccess($this->levelId), $keysTable),
            new CnWithTable(new CnDateStartBetween($this->range), $keysTable),
        ]);
        $from = new TermKeysFromWithUser(new STermKeysFrom($keysTable), $userMetaTable);
        $selection = new UserGotAccessDuringPeriod($this->levels, $this->range);
        $userIn = $keysTable->withAlias("%s.user_id IN({$selection->idsRequest("ga$tablePrefix")})");

        return $from->clause() . PHP_EOL . "WHERE $userIn AND {$filter->toWhere()}";
    }

    public function idsRequest(string $tablePrefix = ''): string
    {
        return "SELECT DISTINCT tk$tablePrefix.user_id {$this->request($tablePrefix)}";
    }
}
